@@ -2,8 +2,6 @@
 Generate and compile C modules for Python.
 
 """
-
-
 import atexit
 import textwrap
 import six.moves.cPickle as pickle
@@ -23,11 +21,13 @@ import warnings
 import numpy.distutils
 
 import theano
-from theano.compat import PY3, decode, decode_iter
-from six import b, BytesIO, StringIO, string_types, iteritems
-from six.moves import xrange
-from theano.gof.utils import flatten
+
+from six import b, BytesIO, StringIO, string_types
+
+
 from theano import config
+from theano.compat import PY3, decode, decode_iter
+from theano.gof.utils import flatten
 from theano.gof.utils import hash_from_code
 from theano.misc.windows import subprocess_Popen, output_subprocess_Popen
 
@@ -65,8 +65,8 @@ def debug_counter(name, every=1):
 
     This is a utility function one may use when debugging.
 
-    Example
-    -------
+    Examples
+    --------
     debug_counter('I want to know how often I run this line')
 
     """
@@ -585,7 +585,7 @@ class KeyData(object):
                 pass
         if do_manual_check:
             to_del = []
-            for key, key_entry in iteritems(entry_from_key):
+            for key, key_entry in entry_from_key.items():
                 if key_entry == entry:
                     to_del.append(key)
             for key in to_del:
@@ -1767,10 +1767,7 @@ def std_lib_dirs_and_libs():
     else:
         if platform.python_implementation() == "PyPy":
             # Assume Linux (note: Ubuntu doesn't ship this .so)
-            if sys.version_info < (3,):
-                libname = "pypy-c"
-            else:
-                libname = "pypy3-c"
+            libname = "pypy3-c"
             # Unfortunately the only convention of this .so is that it appears
             # next to the location of the interpreter binary.
             libdir = os.path.dirname(os.path.realpath(sys.executable))
@@ -1989,8 +1986,8 @@ class Compiler(object):
 
 def try_march_flag(flags):
     """
-        Try to compile and run a simple C snippet using current flags.
-        Return: compilation success (True/False), execution success (True/False)
+    Try to compile and run a simple C snippet using current flags.
+    Return: compilation success (True/False), execution success (True/False)
     """
     test_code = textwrap.dedent(
         """\
@@ -2270,7 +2267,7 @@ class GCC_compiler(Compiler):
                 default_detected_flag = []
                 march_flags_to_try = ["corei7-avx", "corei7", "core2"]
 
-                for m_ in xrange(len(GCC_compiler.march_flags)):
+                for m_ in range(len(GCC_compiler.march_flags)):
                     march_flag = GCC_compiler.march_flags[m_]
                     if "march" in march_flag:
                         march_ind = m_
@@ -2353,7 +2350,7 @@ class GCC_compiler(Compiler):
             # redefinition for recent CPython versions (>=2.7.16 and >=3.7.3).
             # The following nullifies that redefinition, if it is found.
             python_version = sys.version_info[:3]
-            if python_version < (2, 7, 16) or (3,) <= python_version < (3, 7, 3):
+            if (3,) <= python_version < (3, 7, 3):
                 config_h_filename = distutils.sysconfig.get_config_h_filename()
                 try:
                     with open(config_h_filename) as config_h:
