@@ -4,17 +4,22 @@ They all allow different way to print a graph or the result of an Op
 in a graph(Print Op)
 """
 
-from copy import copy
 import logging
 import os
 import sys
 import hashlib
 
 import numpy as np
-from six import string_types, integer_types, iteritems
-from six.moves import StringIO, reduce
 
 import theano
+
+from functools import reduce
+
+from copy import copy
+
+from six import string_types, integer_types
+from six.moves import StringIO
+
 from theano import gof
 from theano import config
 from theano.gof import Op, Apply
@@ -307,7 +312,7 @@ def _print_fn(op, xin):
 
 
 class Print(Op):
-    """ This identity-like Op print as a side effect.
+    """This identity-like Op print as a side effect.
 
     This identity-like Op has the side effect of printing a message
     followed by its inputs when it runs. Default behaviour is to print
@@ -623,7 +628,7 @@ class PPrinter:
         pprinter = self.clone_assign(
             lambda pstate, r: r.name is not None and r is not current, leaf_printer
         )
-        inv_updates = dict((b, a) for (a, b) in iteritems(updates))
+        inv_updates = dict((b, a) for (a, b) in updates.items())
         i = 1
         for node in gof.graph.io_toposort(
             list(inputs) + updates.keys(), list(outputs) + updates.values()
@@ -686,7 +691,11 @@ else:
     special = dict(middle_dot="\u00B7", big_sigma="\u03A3")
 
     greek = dict(
-        alpha="\u03B1", beta="\u03B2", gamma="\u03B3", delta="\u03B4", epsilon="\u03B5",
+        alpha="\u03B1",
+        beta="\u03B2",
+        gamma="\u03B3",
+        delta="\u03B4",
+        epsilon="\u03B5",
     )
 
 
@@ -972,7 +981,7 @@ def pydotprint(
         astr, aid = apply_name(node)
 
         use_color = None
-        for opName, color in iteritems(colorCodes):
+        for opName, color in colorCodes.items():
             if opName in node.op.__class__.__name__:
                 use_color = color
 
@@ -1174,9 +1183,9 @@ def pydotprint(
 
 
 class _TagGenerator:
-    """ Class for giving abbreviated tags like to objects.
-        Only really intended for internal use in order to
-        implement min_informative_st """
+    """Class for giving abbreviated tags like to objects.
+    Only really intended for internal use in order to
+    implement min_informative_st"""
 
     def __init__(self):
         self.cur_tag_number = 0
