@@ -17,7 +17,7 @@ import warnings
 
 import numpy as np
 from six import integer_types, string_types
-from six.moves import xrange
+
 
 import theano
 from theano import Op, Apply, shared, config, Variable
@@ -423,7 +423,7 @@ class mrg_uniform(mrg_uniform_base):
 
         err_orig = np.seterr(over="ignore")
         try:
-            for i in xrange(n_elements):
+            for i in range(n_elements):
                 sample = mrg_next_value(
                     rstate[i % n_streams],
                     rstate[i % n_streams],
@@ -827,7 +827,7 @@ class MRG_RandomStreams(object):
         f.input_storage[2].storage[0] = M1
         f.input_storage[3].storage[0] = A2p72
         f.input_storage[5].storage[0] = M2
-        for i in xrange(1, n_streams):
+        for i in range(1, n_streams):
             # Inline the following call to bypass Python overhead
             # rval[i] = ff_2p72(rval[i - 1])
             v = rval[i - 1]
