@@ -12,7 +12,7 @@ See especially conv2d().
 import logging
 
 import numpy as np
-from six.moves import xrange
+
 import warnings
 
 import theano
@@ -114,7 +114,7 @@ def conv2d(
     # accept Constant value for image_shape and filter_shape.
     if image_shape is not None:
         image_shape = list(image_shape)
-        for i in xrange(len(image_shape)):
+        for i in range(len(image_shape)):
             if image_shape[i] is not None:
                 try:
                     image_shape[i] = get_scalar_constant_value(
@@ -130,7 +130,7 @@ def conv2d(
                 image_shape[i] = int(image_shape[i])
     if filter_shape is not None:
         filter_shape = list(filter_shape)
-        for i in xrange(len(filter_shape)):
+        for i in range(len(filter_shape)):
             if filter_shape[i] is not None:
                 try:
                     filter_shape[i] = get_scalar_constant_value(
@@ -590,13 +590,21 @@ class ConvOp(OpenMPOp):
 
         self.outshp = get_conv_output_shape(
             (None,) + self.imshp_logical,
-            (None, None,) + self.kshp_logical,
+            (
+                None,
+                None,
+            )
+            + self.kshp_logical,
             output_mode,
             (dx, dy),
         )[2:]
         self.fulloutshp = get_conv_output_shape(
             (None,) + self.imshp_logical,
-            (None, None,) + self.kshp_logical,
+            (
+                None,
+                None,
+            )
+            + self.kshp_logical,
             output_mode,
             (1, 1),
         )[2:]
@@ -633,7 +641,7 @@ class ConvOp(OpenMPOp):
                 else:
                     time_unroll_patch = self.speed_unroll_patch_noshape[mode_idx]
                 time_unroll_batch_kern = 9999999
-                for i in xrange(len(self.speed_unroll_batch_kern)):
+                for i in range(len(self.speed_unroll_batch_kern)):
                     if (
                         bsize % self.speed_unroll_batch_kern[i][0] == 0
                         and nkern % self.speed_unroll_batch_kern[i][1] == 0
@@ -782,7 +790,11 @@ class ConvOp(OpenMPOp):
         # infer output shape from what we have
         res = get_conv_output_shape(
             (bsize,) + tuple(imshp),
-            (nkern, None,) + tuple(kshp),
+            (
+                nkern,
+                None,
+            )
+            + tuple(kshp),
             self.out_mode,
             (self.dx, self.dy),
         )
@@ -869,13 +881,32 @@ class ConvOp(OpenMPOp):
         else:
             fulloutshp = get_conv_output_shape(
                 (None,) + imshp_logical,
-                (None, None,) + kshp_logical,
+                (
+                    None,
+                    None,
+                )
+                + kshp_logical,
                 self.out_mode,
                 (1, 1),
             )[2:]
 
-        if z[0] is None or z[0].shape != (bsize, nkern,) + fulloutshp:
-            z[0] = np.zeros((bsize, nkern,) + fulloutshp, dtype=img2d.dtype)
+        if (
+            z[0] is None
+            or z[0].shape
+            != (
+                bsize,
+                nkern,
+            )
+            + fulloutshp
+        ):
+            z[0] = np.zeros(
+                (
+                    bsize,
+                    nkern,
+                )
+                + fulloutshp,
+                dtype=img2d.dtype,
+            )
         zz = z[0]
 
         stacklen = imshp[0]
@@ -918,10 +949,10 @@ class ConvOp(OpenMPOp):
 
         with warnings.catch_warnings():
             warnings.simplefilter("ignore", np.ComplexWarning)
-            for b in xrange(bsize):
-                for n in xrange(nkern):
+            for b in range(bsize):
+                for n in range(nkern):
                     zz[b, n, ...].fill(0)
-                    for im0 in xrange(stacklen):
+                    for im0 in range(stacklen):
                         # some cast generates a warning here
                         zz[b, n, ...] += _convolve2d(
                             img2d[b, im0, ...],
@@ -951,12 +982,12 @@ class ConvOp(OpenMPOp):
                 img2d = img2d2
             # N_image_shape = image_data.shape
 
-            for b in xrange(bsize):
-                for n in xrange(nkern):
+            for b in range(bsize):
+                for n in range(nkern):
                     zz[b, n, ...].fill(0)
-                    for im0 in xrange(stacklen):
-                        for row in xrange(0, zz.shape[2], self.dx):
-                            for col in xrange(0, zz.shape[3], self.dy):
+                    for im0 in range(stacklen):
+                        for row in range(0, zz.shape[2], self.dx):
+                            for col in range(0, zz.shape[3], self.dy):
                                 zz[b, n, row, col] += (
                                     img2d[
                                         b, im0, row : row + kshp[0], col : col + kshp[1]
@@ -1130,8 +1161,7 @@ using namespace std;
         )
 
     def use_blas(self):
-        """ Return True if we will generate code that use gemm.
-        """
+        """Return True if we will generate code that use gemm."""
         # the gemm version only support that case
         if self.out_mode == "valid" and self.dx == 0 and self.dy == 0:
             # We use a faster version in those case.
@@ -2021,7 +2051,7 @@ def gen_conv_code_unroll_batch_kern(d, unroll_bsize=1, unroll_ksize=1):
 
     def my_dup(st, size):
         s = ""
-        for i in xrange(size):
+        for i in range(size):
             d["unroll_iter"] = i
             s += st % d
         return s + "\n"
@@ -2029,9 +2059,9 @@ def gen_conv_code_unroll_batch_kern(d, unroll_bsize=1, unroll_ksize=1):
     def my_dup2(st):
         s = ""
         iter = 0
-        for i in xrange(unroll_bsize):
+        for i in range(unroll_bsize):
             d["unroll_biter"] = i
-            for j in xrange(unroll_ksize):
+            for j in range(unroll_ksize):
                 d["unroll_kiter"] = j
                 d["unroll_iter"] = iter
                 iter += 1
