@@ -1,21 +1,24 @@
 import sys
-from textwrap import dedent
 import warnings
 import logging
 
 import numpy as np
-from six import integer_types
-from six.moves import xrange
 
 import theano
-from theano.compat import izip
-from theano.gradient import DisconnectedType
-from theano import gof
+
+from textwrap import dedent
+
+from itertools import groupby, chain
+from collections.abc import Iterable
+
+from six import integer_types
+
+from theano import gof, scalar as scal, config
 from theano.gof import Apply, hashtype, Op, Type, MethodNotDefined, ParamsType
+from theano.gradient import DisconnectedType
 from theano.printing import pprint
-from theano import scalar as scal
-from theano.tensor.basic import alloc
 from theano.tensor.basic import (
+    alloc,
     addbroadcast,
     clip,
     get_scalar_constant_value,
@@ -23,17 +26,12 @@ from theano.tensor.basic import (
     NotScalarConstantError,
 )
 from theano.tensor.elemwise import DimShuffle
+from theano.tensor.inc_code import inc_code
+from theano.tensor.extra_ops import broadcast_shape
 from theano.tensor.type_other import NoneConst, SliceType, NoneTypeT, make_slice
-from theano import config
-from theano.compat import Iterable
-
-from .inc_code import inc_code
 
 _logger = logging.getLogger("theano.tensor.subtensor")
 
-# Do a lazy import of the sparse module
-sparse_module_ref = None
-
 
 class AdvancedIndexingError(TypeError):
     """
@@ -53,28 +51,25 @@ class AdvancedBooleanIndexingError(TypeError):
     pass
 
 
-##########
-# Helpful functions to deal with Subtensor and IncSubtensor
-##########
+def as_index_constant(a):
+    """Convert Python literals to Theano constants--when possible--in Subtensor arguments.
 
-
-def make_constant(args):
+    This will leave `Variable`s untouched.
     """
-    Convert python litterals to theano constants in subtensor arguments.
-
-    """
-
-    def conv(a):
-        if a is None:
-            return a
-        elif isinstance(a, slice):
-            return slice(conv(a.start), conv(a.stop), conv(a.step))
-        elif isinstance(a, (integer_types, np.integer)):
-            return scal.ScalarConstant(scal.int64, a)
-        else:
-            return a
-
-    return tuple(map(conv, args))
+    if a is None:
+        return a
+    elif isinstance(a, slice):
+        return slice(
+            as_index_constant(a.start),
+            as_index_constant(a.stop),
+            as_index_constant(a.step),
+        )
+    elif isinstance(a, (integer_types, np.integer)):
+        return scal.ScalarConstant(scal.int64, a)
+    elif not isinstance(a, theano.tensor.Variable):
+        return theano.tensor.as_tensor(a)
+    else:
+        return a
 
 
 def get_idx_list(inputs, idx_list, get_count=False):
@@ -112,7 +107,8 @@ def get_idx_list(inputs, idx_list, get_count=False):
 
 
 def get_canonical_form_slice(theslice, length):
-    """
+    """Convert slices to canonical form.
+
     Given a slice [start:stop:step] transform it into a canonical form
     that respects the conventions imposed by python and numpy.
 
@@ -277,41 +273,167 @@ def get_canonical_form_slice(theslice, length):
         return value, 1
 
 
+def range_len(slc):
+    """Length of a `range` object.
+
+    Adapted from CPython.
+
+    """
+    from theano.tensor import switch, and_, lt, gt
+
+    start, stop, step = tuple(
+        as_index_constant(a) for a in [slc.start, slc.stop, slc.step]
+    )
+    return switch(
+        and_(gt(step, 0), lt(start, stop)),
+        1 + (stop - 1 - start) // step,
+        switch(
+            and_(lt(step, 0), gt(start, stop)),
+            1 + (start - 1 - stop) // (-step),
+            scal.ScalarConstant(scal.int64, 0),
+        ),
+    )
+
+
+def slice_len(slc, n):
+    """Compute the length of a slice for an array of a given length.
+
+    We're essentially computing `len(range(*slc.indices(n)))`.
+
+    """
+    # TODO: Do we need to do this or should we expect `slc` to
+    # already be canonicalized?
+    canon_slc, _ = get_canonical_form_slice(slc, n)
+    return range_len(canon_slc)
+
+
+def is_basic_idx(idx):
+    """Determine if an index is of the NumPy basic type.
+
+    XXX: This only checks a single index, so an integers is *not* considered a
+    basic index, because--depending on the other indices its used with--an
+    integer can indicate advanced indexing.
+
+    """
+    return isinstance(idx, (slice, type(None))) or isinstance(
+        getattr(idx, "type", None), (SliceType, NoneTypeT)
+    )
+
+
+def basic_shape(shape, indices):
+    """Computes the shape resulting from basic NumPy indexing.
+
+    Basic indices are either `slice`s or `None`s.
+    `Ellipsis` are not supported here; convert them to `slice`s first.
+
+    Parameters
+    ----------
+    shape: Tuple[int]
+        The shape of the array being indexed
+    indices: Sequence[Or[slice, NoneType]]
+        A sequence of basic indices used to index an array.
+
+    """
+    res_shape = ()
+    for idx, n in zip(indices, shape):
+        if isinstance(idx, slice):
+            res_shape += (slice_len(idx, n),)
+        elif isinstance(getattr(idx, "type", None), SliceType):
+            if idx.owner:
+                idx_inputs = idx.owner.inputs
+            else:
+                idx_inputs = (None,)
+            res_shape += (slice_len(slice(*idx_inputs), n),)
+        elif idx is None:
+            res_shape += (scal.ScalarConstant(scal.int64, 1),)
+        elif isinstance(getattr(idx, "type", None), NoneTypeT):
+            res_shape += (scal.ScalarConstant(scal.int64, 1),)
+        else:
+            raise ValueError("Invalid index type: {}".format(idx))
+    return res_shape
+
+
+def group_indices(indices):
+    """Group indices sequentially by whether or not they're basic or advanced.
+
+    Returns
+    -------
+    Tuple[Boolean, List[Tuple[Integer, Any]]]
+        The boolean indicates whether or not the group is a set of basic
+        indices.  The list contains the contiguous set of indices paired with their
+        corresponding dimension number in the array being indexed.
+    """
+    idx_groups = []
+    dim_num = -1
+    for basic, grp_indices in groupby(indices, key=is_basic_idx):
+        enum_grp_indices = []
+        for idx in grp_indices:
+            # We "zip" the dimension number to each index, which means we can't
+            # count indices that add new axes
+            if (idx is not None) and not isinstance(
+                getattr(idx, "type", None), NoneTypeT
+            ):
+                dim_num += 1
+
+            enum_grp_indices.append((dim_num, idx))
+
+        idx_groups.append((basic, enum_grp_indices))
+
+    return idx_groups
+
+
+def indexed_result_shape(array_shape, indices, indices_are_shapes=False):
+    """Compute the symbolic shape resulting from `a[indices]` for `a.shape == array_shape`.
+
+    This function uses NumPy's basic and advanced indexing logic.  It can also
+    handle combinations of advanced and basic indices.
+
+    Parameters
+    ----------
+    array_shape: Tuple[Variable]
+        Shape of the array being indexed.
+    indices: Sequence[Union[TensorVariable, Tuple[Union[None, slice, Variable]]]]
+        Either the indices themselves or the shapes of each index--depending
+        on the value of `indices_are_shapes`.
+    indices_are_shapes: bool (Optional)
+        Indicates whether or not the `indices` contains shape tuples instead of
+        the actual index arrays.  If you use this approach, make sure that the
+        broadcastable dimensions are (scalar) constants with the value `1`, or `1`
+        exactly.
+    """
+    res_shape = ()
+
+    remaining_dims = range(theano.tensor.basic.get_vector_length(array_shape))
+    idx_groups = group_indices(indices)
+
+    if len(idx_groups) > 2 or len(idx_groups) > 1 and not idx_groups[0][0]:
+        # Bring adv. index groups to the front and merge each group
+        idx_groups = sorted(idx_groups, key=lambda x: x[0])
+        idx_groups = groupby(
+            chain.from_iterable(d_idx for _, d_idx in idx_groups),
+            key=lambda x: is_basic_idx(x[1]),
+        )
+
+    for basic, grp_dim_indices in idx_groups:
+        dim_nums, grp_indices = zip(*grp_dim_indices)
+        remaining_dims = tuple(dim for dim in remaining_dims if dim not in dim_nums)
+
+        if basic:
+            grp_shapes = tuple(array_shape[dim] for dim in dim_nums)
+            res_shape += basic_shape(grp_shapes, grp_indices)
+        else:
+            res_shape += broadcast_shape(
+                *grp_indices, arrays_are_shapes=indices_are_shapes
+            )
+
+    res_shape += tuple(array_shape[dim] for dim in remaining_dims)
+
+    return res_shape
+
+
 class Subtensor(Op):
-    """
-    Return a subtensor view.
+    """Basic NumPy indexing operator."""
 
-    The inputs array is the tensor x, followed by scalar integer types.
-    TODO: WRITEME: how are the scalar integer variables formatted?
-
-    This class uses a relatively complex internal representation of the inputs
-    to remember how the input tensor x should be sliced.
-
-    idx_list: instance variable TODO: WRITEME: is this a list or a tuple?
-                                        (old docstring gives two conflicting
-                                        descriptions)
-              elements are either integers, theano scalar types, or slices.
-              one element per "explicitly named dimension"
-                TODO: WRITEME: what is an "explicitly named dimension" ?
-
-              if integer:
-                  indexes into the inputs array
-              if slice:
-                  start/stop/step members of each slice are integer indices
-                  into the inputs array or None
-                  integer indices be actual integers or theano scalar types
-
-    Note that the idx_list defines the Op, so two Subtensor instances are
-    considered to be different Ops if they have different idx_list fields.
-    This means that the entries in it are theano Types, not theano Variables.
-
-    @todo: add support for advanced tensor indexing (in Subtensor_dx too).
-
-    """
-
-    e_subslice = "nested slicing is not supported"
-    e_indextype = "Invalid index type or slice for Subtensor"
-    debug = 0
     check_input = False
     view_map = {0: [0]}
     _f16_ok = True
@@ -359,27 +481,29 @@ class Subtensor(Op):
             when would that happen?
 
         """
-        invalid_scal_types = [scal.float64, scal.float32, scal.float16]
-        scal_types = [scal.int64, scal.int32, scal.int16, scal.int8]
-        tensor_types = [
+        invalid_scal_types = (scal.float64, scal.float32, scal.float16)
+        scal_types = (scal.int64, scal.int32, scal.int16, scal.int8)
+        tensor_types = (
             theano.tensor.lscalar,
             theano.tensor.iscalar,
             theano.tensor.wscalar,
             theano.tensor.bscalar,
-        ]
-        invalid_tensor_types = [
+        )
+        invalid_tensor_types = (
             theano.tensor.fscalar,
             theano.tensor.dscalar,
             theano.tensor.cscalar,
             theano.tensor.zscalar,
-        ]
+        )
 
         if (
             isinstance(entry, (np.ndarray, theano.tensor.Variable))
             and hasattr(entry, "dtype")
             and entry.dtype == "bool"
         ):
-            raise AdvancedBooleanIndexingError(Subtensor.e_indextype, entry)
+            raise AdvancedBooleanIndexingError(
+                "Invalid index type or slice for Subtensor"
+            )
 
         if isinstance(entry, gof.Variable) and (
             entry.type in invalid_scal_types or entry.type in invalid_tensor_types
@@ -433,7 +557,7 @@ class Subtensor(Op):
                 "Python scalar in idx_list." "Please report this error to theano-dev."
             )
         else:
-            raise AdvancedIndexingError(Subtensor.e_indextype, entry)
+            raise AdvancedIndexingError("Invalid index type or slice for Subtensor")
 
     def get_constant_idx(
         self, inputs, allow_partial=False, only_process_constants=False, elemwise=True
@@ -527,7 +651,7 @@ class Subtensor(Op):
             raise IndexError(
                 "Not enough inputs to fill in the Subtensor template.", inputs, idx_list
             )
-        for input, expected_type in izip(inputs, input_types):
+        for input, expected_type in zip(inputs, input_types):
             if input.type != expected_type:
                 raise TypeError(
                     "Wrong type for Subtensor template. Expected %s, got %s."
@@ -539,7 +663,7 @@ class Subtensor(Op):
             slice(None, None, None)
         ] * (x.type.ndim - len(idx_list))
         broadcastable = []
-        for i, (p, bc) in enumerate(izip(padded, x.type.broadcastable)):
+        for i, (p, bc) in enumerate(zip(padded, x.type.broadcastable)):
             if isinstance(p, slice):
                 if bc:
                     start = p.start
@@ -585,7 +709,7 @@ class Subtensor(Op):
             len(xshp) - len(self.idx_list)
         )
         i = 0
-        for idx, xl in izip(padded, xshp):
+        for idx, xl in zip(padded, xshp):
             if isinstance(idx, slice):
                 # If it is the default (None, None, None) slice, or a variant,
                 # the shape will be xl
@@ -1147,7 +1271,7 @@ def inc_subtensor(
         )
 
     dim_offset = x.ndim - y.ndim
-    for dim in xrange(y.ndim):
+    for dim in range(y.ndim):
         if x.broadcastable[dim + dim_offset] and not y.broadcastable[dim]:
             # It is acceptable to try to increment a subtensor with a
             # broadcastable dim with a tensor that is not broadcastable
@@ -1259,7 +1383,7 @@ def inc_subtensor(
         if y.ndim > 0:
             # This if is needed to prevent some useless warning about
             # old code bug.
-            expanded_y = alloc(y, *[x.shape[i] for i in xrange(x.ndim)])
+            expanded_y = alloc(y, *[x.shape[i] for i in range(x.ndim)])
             flattened_y = expanded_y.reshape(inner_x.shape)
         else:
             flattened_y = y
@@ -1398,7 +1522,7 @@ class IncSubtensor(Op):
             raise IndexError(
                 "Not enough inputs to fill in the Subtensor template.", inputs, idx_list
             )
-        for input, expected_type in izip(inputs, input_types):
+        for input, expected_type in zip(inputs, input_types):
             if input.type != expected_type:
                 raise TypeError(
                     "Wrong type for Subtensor template. Expected %s, got %s."
@@ -1417,21 +1541,7 @@ class IncSubtensor(Op):
 
         def convert(entry):
             if isinstance(entry, gof.Type):
-                rval = indices.pop()
-                if sys.version_info < (2, 5):
-                    # Before Python 2.5, PySlice_GetIndicesEx requires
-                    # Python int to be passed.
-                    rval_ = int(rval)
-                    if rval_ != rval:
-                        raise IndexError(
-                            (
-                                "Invalid value for indexing: %s. "
-                                "That value may be too big."
-                            )
-                            % rval
-                        )
-                    return rval_
-                return rval
+                return indices.pop()
             elif isinstance(entry, slice):
                 return slice(
                     convert(entry.start), convert(entry.stop), convert(entry.step)
@@ -1763,7 +1873,7 @@ def _sum_grad_over_bcasted_dims(x, gx):
         x_broad = (True,) * x_dim_added + x.broadcastable
         assert sum(gx.broadcastable) < sum(x_broad)
         axis_to_sum = []
-        for i in xrange(gx.ndim):
+        for i in range(gx.ndim):
             if gx.broadcastable[i] is False and x_broad[i] is True:
                 axis_to_sum.append(i)
             elif gx.broadcastable[i] is True and x_broad[i] is False:
@@ -1776,21 +1886,13 @@ def _sum_grad_over_bcasted_dims(x, gx):
         gx = gx.sum(axis=axis_to_sum, keepdims=True)
         if gx.ndim != x.ndim:
             assert gx.ndim > x.ndim
-            for i in xrange(x_dim_added):
+            for i in range(x_dim_added):
                 assert gx.broadcastable[i]
             gx = gx.dimshuffle(*list(range(x_dim_added, gx.ndim)))
         assert gx.broadcastable == x.broadcastable
     return gx
 
 
-#########################
-# Advanced indexing
-#########################
-#
-# Should reproduce numpy's behaviour, see url:
-# docs.scipy.org/doc/numpy/reference/arrays.indexing.html#advanced-indexing
-
-
 class AdvancedSubtensor1(Op):
     """
     Implement x[ilist] where ilist is a vector of integers.
@@ -1858,7 +1960,6 @@ class AdvancedSubtensor1(Op):
         return rval
 
     def grad(self, inputs, grads):
-        global sparse_module_ref
         x, ilist = inputs
         (gz,) = grads
         assert len(inputs) == 2
@@ -1868,10 +1969,8 @@ class AdvancedSubtensor1(Op):
                     "AdvancedSubtensor1: you can't take the sparse grad"
                     " from a tensor with ndim != 2. ndim is " + str(x.type.ndim)
                 )
-            if sparse_module_ref is None:
-                import theano.sparse as sparse_module_ref
 
-            rval1 = [sparse_module_ref.construct_sparse_from_list(x, gz, ilist)]
+            rval1 = [theano.sparse.construct_sparse_from_list(x, gz, ilist)]
         else:
             if x.dtype in theano.tensor.discrete_dtypes:
                 # The output dtype is the same as x
@@ -2203,45 +2302,6 @@ def as_index_variable(idx):
     return idx
 
 
-def adv_index_broadcastable_pattern(a, idx):
-    """
-    This function is only used to determine the broadcast pattern for
-    AdvancedSubtensor output variable.
-
-    For this, we make a fake ndarray and a fake idx and call use ask numpy
-    the output. From this, we find the output broadcast pattern.
-
-    """
-
-    def replace_slice(v):
-        if isinstance(v, gof.Apply):
-            if len(v.outputs) != 1:
-                raise ValueError(
-                    "It is ambiguous which output of a multi-output Op has"
-                    " to be fetched.",
-                    v,
-                )
-            else:
-                v = v.outputs[0]
-
-        if NoneConst.equals(v):
-            return None
-        if isinstance(v.type, SliceType):
-            return slice(None, None)
-
-        if v.dtype == "bool":
-            return np.ones((2,) * v.ndim, v.dtype)
-        else:
-            return np.zeros((2,) * v.ndim, int)
-
-    newidx = tuple(map(replace_slice, idx))
-
-    # 2 - True = 1; 2 - False = 2
-    fakeshape = [2 - bc for bc in a.broadcastable]
-    retshape = np.empty(fakeshape)[newidx].shape
-    return tuple([dim == 1 for dim in retshape])
-
-
 def check_advanced_indexing_dimensions(input, idx_list):
     """
     This function checks if the index list in idx_list is correct.
@@ -2256,7 +2316,7 @@ def check_advanced_indexing_dimensions(input, idx_list):
             # skip, does not count as an input dimension
             pass
         elif isinstance(index, np.ndarray) and index.dtype == "bool":
-            for i in xrange(index.ndim):
+            for i in range(index.ndim):
                 if index.shape[i] != input.shape[dim_seen + i]:
                     raise IndexError(
                         "boolean index did not match indexed array "
@@ -2288,23 +2348,47 @@ def check_and_reject_bool(args_el):
 
 
 class BaseAdvancedSubtensor(Op):
-    """
-    Abstract base class for AdvancedSubtensor and AdvancedBooleanSubtensor.
+    """Abstract base class for AdvancedSubtensor and AdvancedBooleanSubtensor.
+
     Implements advanced indexing with boolean masks.
 
+    Should be used by __getitem__ and __getslice__, as follows:
+        - AdvancedSubtensor()(self, *args) or
+        - AdvancedBooleanSubtensor()(self, *args), if args contain advanced indices
+
     """
 
-    # Should be used by __getitem__ and __getslice__, as follows:
-    # AdvancedSubtensor()(self, *args) or
-    # AdvancedBooleanSubtensor()(self, *args),
-    # if args contains and advanced indexing pattern
     __props__ = ()
 
-    def make_node(self, x, *index):
+    def make_node(self, x, *index, is_boolean=False):
         x = theano.tensor.as_tensor_variable(x)
-
         index = tuple(map(as_index_variable, index))
-        bcast = adv_index_broadcastable_pattern(x, index)
+
+        # We only want the broadcast information, and we don't need recursive
+        # `Subtensor` calls, so we create a fake symbolic shape tuple and
+        # identify the broadcast dimensions from the shape result of this
+        # entire subtensor operation.
+        fake_shape = tuple(
+            theano.tensor.tensor(dtype="int64", broadcastable=()) if not bcast else 1
+            for bcast in x.broadcastable
+        )
+
+        bcast_index = index
+        if is_boolean:
+            bcast_index = tuple(
+                chain.from_iterable(
+                    theano.tensor.basic.nonzero(idx)
+                    if getattr(idx, "ndim", 0) > 0
+                    else (idx,)
+                    for idx in bcast_index
+                )
+            )
+
+        bcast = [
+            getattr(i, "value", i) == 1
+            for i in indexed_result_shape(fake_shape, bcast_index)
+        ]
+
         return gof.Apply(
             self,
             (x,) + index,
@@ -2317,8 +2401,26 @@ class BaseAdvancedSubtensor(Op):
         return self.make_node(eval_points[0], *inputs[1:]).outputs
 
     def infer_shape(self, node, ishapes):
-        # Default case, we don't know
-        raise theano.tensor.basic.ShapeError("case not implemented")
+        indices = node.inputs[1:]
+        index_shapes = list(ishapes[1:])
+        for i, idx in enumerate(indices):
+            if (
+                isinstance(idx, (np.bool_, bool))
+                or getattr(idx, "dtype", None) == "bool"
+            ):
+                raise theano.tensor.basic.ShapeError(
+                    "Shape inference for boolean indices is not implemented"
+                )
+            # The `ishapes` entries for `SliceType`s will be None, and
+            # we need to give `indexed_result_shape` the actual slices.
+            if isinstance(getattr(idx, "type", None), SliceType):
+                index_shapes[i] = idx
+
+        res_shape = indexed_result_shape(
+            ishapes[0], index_shapes, indices_are_shapes=True
+        )
+        assert node.outputs[0].ndim == len(res_shape)
+        return [[s for s in res_shape]]
 
     def perform(self, node, inputs, out_):
         (out,) = out_
@@ -2348,34 +2450,10 @@ class AdvancedSubtensor(BaseAdvancedSubtensor):
 
     """
 
-    # Should be used by __getitem__ and __getslice__, as follows:
-    # AdvancedSubtensor()(self, *args),
-    # if args contains and advanced indexing pattern
-
     def make_node(self, x, *index):
         check_and_reject_bool(index)
         return super(AdvancedSubtensor, self).make_node(x, *index)
 
-    def infer_shape(self, node, ishapes):
-        # Really special case
-        if len(ishapes) == 3:
-            xshp, ind1shp, ind2shp = ishapes
-            if (
-                len(xshp) == 2
-                and ind1shp is not None
-                and len(ind1shp) == 1
-                and ind2shp is not None
-                and len(ind2shp) == 1
-            ):
-                # if the graph is correct, we can assume ind1shp[0] and
-                # ind2shp[0] will have the same value.
-                # Try to return the one closest to the graph input.
-                if node.inputs[2].owner is None:
-                    return [ind2shp]
-                else:
-                    return [ind1shp]
-        return super(AdvancedSubtensor, self).infer_shape(node, ishapes)
-
     def grad(self, inputs, grads):
         (gz,) = grads
         x = inputs[0]
@@ -2401,9 +2479,8 @@ class AdvancedBooleanSubtensor(BaseAdvancedSubtensor):
 
     """
 
-    # Should be used by __getitem__ and __getslice__, as follows:
-    # AdvancedBooleanSubtensor()(self, *args),
-    # if args contains and advanced indexing pattern with boolean masks
+    def make_node(self, x, *index):
+        return super().make_node(x, *index, is_boolean=True)
 
     def grad(self, inputs, grads):
         (gz,) = grads
@@ -2575,6 +2652,27 @@ advanced_boolean_set_subtensor = AdvancedBooleanIncSubtensor(set_instead_of_inc=
 
 
 def take(a, indices, axis=None, mode="raise"):
+    """Take elements from an array along an axis.
+
+    When axis is not None, this function does the same thing as "fancy"
+    indexing (indexing arrays using arrays); however, it can be easier to use
+    if you need elements along a given axis. A call such as
+    ``np.take(arr, indices, axis=3)`` is equivalent to
+    ``arr[:,:,:,indices,...]``.
+
+    See `np.take`
+
+    Parameters
+    ----------
+    a : Tensor
+        The source array.
+    indices : Tensor, ndarray, list, tuple
+        The indices of the values to extract.
+    axis : int, optional
+        The axis over which to select values. By default, the flattened
+        input array is used.
+
+    """
     a = theano.tensor.as_tensor_variable(a)
     indices = theano.tensor.as_tensor_variable(indices)
     # Reuse advanced_subtensor1 if indices is a vector
